<?php
// $Id: kdbrunidentifier.php 973 2009-08-19 09:40:28Z s_voec01 $
// Author: Sebastian Voecking <sebastian.voecking@uni-muenster.de>

ADEI::RequireClass('KDBConstants', TRUE);
ADEI::RequireClass('KDBRunIdentifier', TRUE);

class KDBFileIdentifier
{
    private $fRun;
    private $fType;
    private $fHardware = kHardwareInvalid;
    private $fName = '';

    function __construct(KDBRunIdentifier $run, $type)
    {
        $this->fRun = $run;
        $this->fType = $type;
    }

    function GetFilename()
    {
        $run = $this->fRun->GetRun();
        $subrun = $this->fRun->GetSubrun();
        $run_name = $this->fRun->GetName();
        $hardware = $this->GetHardwareName();
        
        switch ($this->fType) {
            case kTypeOrca:
                $filename = "Run$run";
                if ($subrun) {
                    $filename .= "-$subrun";
                }
                return $filename;
                
            case kTypeOrcaRoot:
                $filename = "OR_run$run";
                if ($subrun) {
                    $filename .= "-$subrun";
                }
                $filename .= '.root';
                return $filename;
                
            case kTypeCrun:
                $filename = "crun${run}_";
                switch ($this->fHardware) {
                    case kHardwareUw:
                        $filename .= 'UW';
                        break;
                        
                    case kHardwareIpe3:
                        $filename .= 'IPE3';
                        break;
                        
                    case kHardwareIpe4:
                        $filename .= 'IPE4';
                        break;
                }
                $filename .= '.root';
                return $filename;
                
            case kTypePlot:
                return "${run_name}_{$this->fName}_$hardware.png";
                
            case kTypePreview:
                return "${run_name}_{$this->fName}_${hardware}_pv.png";

            case kTypeXml:
                $filename = '';
                if ($this->fRun->IsValid()) {
                    $filename = $run_name;
                }
                $filename .= $this->fName;

                if ($this->fHardware != kHardwareInvalid) {
                    $filename .= "_$hardware";
                }
                $filename .= '.xml';

                return $filename;

            default:
                return FALSE;
        }
    }

    function SetRun(KDBRunIdentifier $run)
    {
        $this->fRun = $run;
    }

    function GetRun()
    {
        return $this->fRun;
    }

    function SetName($filename)
    {
        $this->fName = $filename;
    }

    function GetName()
    {
        return $this->fName;
    }

    function SetFileType($type)
    {
        $this->fType = $type;
    }

    function GetFileType()
    {
        return $this->fType;
    }

    function SetHardware($type)
    {
        $this->fHardware = $type;
    }

    function GetHardware()
    {
        return $this->fHardware;
    }

    function GetHardwareName()
    {
        return KDBConstants::HardwareName($this->fHardware);
    }
}

?>
