<?php
// $Id: kdbphp.php 8168 2010-07-28 09:04:43Z s_voec01 $
// Author: Sebastian Voecking <sebastian.voecking@uni-muenster.de>

//ADEI::RequireClass("KDBFileServer", true);
ADEI::RequireClass("KDBConfiguration", true);
ADEI::RequireClass("KDBSql", true);
ADEI::RequireClass("KDBXMlWriter", true);

class KDBPhp
{
    private $fSql;
    private $fPathOrca;
    private $fPathOrcaRoot;
    private $fPathCrun;
    private $fPathPlots;
    private $fFileServerRoot;
    
    function __construct()
    {
        $configuration = new KDBConfiguration("/opt/katrin/etc/kdbserver.conf");
        $sql_server = $configuration->GetSqlServer();
        $sql_dbname = $configuration->GetSqlDbName();
        $sql_user = $configuration->GetSqlUser();
        $sql_password = $configuration->GetSqlPassword();
        $this->fSql = new KDBSql($sql_server, $sql_dbname, $sql_user,
                                 $sql_password);

        $this->fPathOrca = $configuration->GetPathOrca();
        $this->fPathOrcaRoot = $configuration->GetPathOrcaRoot();
        $this->fPathCrun = $configuration->GetPathCrun();
        $this->fPathPlots = $configuration->GetPathPlots();
        $this->fFileServerRoot = $configuration->GetFileServerRoot();
    }

    function GetRunDescription(KDBRunIdentifier $run, $subruns)
    {
        $rundesc = $this->fSql->GetRunDescription($run, $subruns);

        $writer = new KDBXmlWriter(); 
        
        $writer->startElement('result');

        $this->WriteRunInfo($writer, $run, $rundesc);

        if ($subruns) {
            $writer->startElement('subruns');

            foreach ($rundesc['subruns'] as $subrun) {
                $this->WriteSubrunInfo($writer, $run, $subrun);
            }

            $writer->endElement();
        }

        $writer->endElement();
        $writer->endDocument();

        return $writer->outputMemory();
    }

    function ListLastRuns($date, $search, $parameters, $page, $limit)
    {
        $writer = new KDBXmlWriter();
        $writer->startElement('result');
        
        if ($search) {
            $filter = $this->ParseSearch($search);
        }

        if (!$filter['parameters']) {
            $filter['parameters'] = $parameters;
        }

        if ($date == "") {
            $writer->startElement('loading');
            $writer->writeAttribute('search', $filter['search']);
            $writer->writeAttribute('parameters', $filter['parameters']);
            $writer->endDocument();

            return $writer->outputMemory();
        }
        
        $total = 0;
        if ($page < 1) $page = 1;

        $filter['date'] = $date;
        $filter['offset'] = ($page - 1) * $limit;
        $filter['limit'] = $limit;

        $runs = $this->fSql->ListRuns($filter, $total);
        $pages = (int) ceil((float) $total / $limit);
        if ($page > $pages) $page = $pages;

        $date_string = $this->DateString($date);
        if ($date_string) {
            $filter_string = $date_string;
        }
        else {
            $filter_string = "All";
        }

        $writer->startElement('runs');
        $writer->writeAttribute('page', $page);
        $writer->writeAttribute('pages', $pages);
        $writer->writeAttribute('parameters', $filter['parameters']);
        $writer->writeAttribute('search', $filter['search']);
        $writer->writeAttribute('filter', $filter_string);

        foreach($runs as $run) {
            $identifier = $this->ConvertRunIdentifier($run['system'],
                                                      $run['number']);
            
            $writer->startElement('run');
            $writer->writeElement('identifier', $identifier);
            $writer->writeElement('start', $run['start']);
            $writer->writeElement('duration', $run['duration']);
            $writer->writeElement('subruns', $run['subruns']);
            $writer->writeElement('comment', $run['comment']);
            $writer->WriteStreams($run['data']);
            $writer->endElement();
        }

        $writer->endDocument();

        return $writer->outputMemory();
    }

    function GetDates($search)
    {
        $filter = $this->ParseSearch($search);
        $dates = $this->fSql->GetDates($filter);
        
        $writer = new XMLWriter();
        $writer->openMemory();
        $writer->setIndent(true);
        $writer->setIndentString(' ');

        $writer->startDocument();
        $writer->startElement('tree');
        $writer->writeAttribute('id', '0');

        $last_year = 0;
        $last_month = 0;
        if ($last_day = end($dates)) {
            $last_year = (int) gmdate('Y', $last_day);
            $last_month = (int) gmdate('n', $last_day);
        }

        $year = 0;
        $month = 0;
        $writer->startElement('item');
        $writer->writeAttribute('text', 'All');
        $writer->writeAttribute('child', '1');
        $writer->writeAttribute('id', 'all');
        $writer->writeAttribute('open', '1');
        foreach ($dates as $date) {
            $date_year = (int) gmdate('Y', $date);
            $date_month = (int) gmdate('n', $date);
            $date_day = gmdate('j', $date);

            if ($date_year != $year) {
                if ($year != 0) {
                    $writer->endElement();
                    $writer->endElement();
                }
                $year = $date_year;
                $writer->startElement('item');
                $writer->writeAttribute('text', $year);
                $writer->writeAttribute('child', '1');
                $writer->writeAttribute('id', $year);

                $month = 0;
            }
            if ($date_month != $month) {
                if ($month != 0) {
                    $writer->endElement();
                }
                $month = $date_month;
                $writer->startElement('item');
                $writer->writeAttribute('text', gmdate('F', $date));
                $writer->writeAttribute('child', '1');
                $writer->writeAttribute('id', gmdate('Ym', $date));
            }
            $writer->startElement('item');
            $writer->writeAttribute('text', $date_day);
            $writer->writeAttribute('id', gmdate('Ymd', $date));
            $writer->writeAttribute('tooltip', gmdate('Y-m-d', $date));
            $writer->endElement();
        }

        $writer->endElement();

        $writer->endDocument();

        return $writer->outputMemory();
    }


    //function GetFileServer()
    //{
    //    try {
    //        return new KDBFileServer(gFileServer_get());
    //    }
    //    catch (Exception $e) {
    //        throw new ADEIException($e->getMessage());
    //    }
    //}

    function PrintFile(KDBFileIdentifier $file)
    {
        $path = $this->GetAbsolutePath($file);
        
        return file_get_contents($path);
    }

    function GetAbsolutePath(KDBFileIdentifier $file)
    {
        return $this->GetDirectory($file) . '/' . $file->GetFilename();
    }

    function FileExists(KDBFileIdentifier $file)
    {
        $path = $this->GetAbsolutePath($file);

        return file_exists($path);
    }

    function HaveRuns($from, $to)
    {
        return $this->fSql->HaveRuns($from, $to);
    }

    function HaveCountrates($from, $to)
    {
        return $this->fSql->HaveCountRates($from, $to);
    }

    function GetSql()
    {
        return $this->fSql;
    }

    function SearchRuns($string)
    {
        $filter = $this->ParseSearch($string);
        return $this->fSql->ListRuns($filter, $total);
    }

    private function ConvertRunIdentifier($system, $number)
    {
        $names = array("kms", "kps", "xxx");

        return sprintf("%s%08d", $names[$system], $number);
    }

    private function ConvertDetector($detector)
    {
        $names = array('none', 'SPD', 'MPD', 'unknown');

        return $name[$detector];
    }

    private function FormatFloat($value, $digits, $scientific = FALSE)
    {
        $format = $scientific ? 'e' : 'f';
        return sprintf("%.$digits$format", $value);
    }

    private function WriteRunInfo(XMLWriter $writer, KDBRunIdentifier $run,
                                  array $rundesc)
    {
        $parameters = $rundesc['parameters'];

        $writer->startElement('run');

        $writer->writeElement('identifier', $run->GetName());
        $writer->writeElement('date', $rundesc['date']);
        $writer->writeElement('start', $rundesc['start']);
        $writer->writeElement('end', $rundesc['end']);
        $string = $this->ConvertDetector((int) $parameters['detector']);
        $writer->writeElement('detector', $string);
        $writer->WriteFloat('detector_x', $parameters['detector_x'], 2);
        $writer->WriteFloat('detector_y', $parameters['detector_y'], 2);
        $writer->WriteFloat('detector_z', $parameters['detector_z'], 2);
        $writer->writeElement('configuration', $rundesc['configuration']);
        $writer->writeElement('orca_version', $rundesc['orca_version']);
        $writer->writeElement('comment', $rundesc['comment']);
        $writer->WriteStreams($rundesc['streams']);

        $this->WriteParameters($writer, $parameters);

        $writer->startElement('plots');
        $file = new KDBFileIdentifier($run, kTypePlot);
        $file->SetHardware(kHardwareIpe3);

        $file->SetName('parameter_time');
        if ($rundesc['parameter'] && $this->FileExists($file)) {
            $writer->startElement('parameter');
            $writer->writeAttribute('parameter', $rundesc['parameter']);
            $writer->WriteImage($file, 'Parameter over time');
            $writer->endElement();
        }

        $file->SetName('events_parameter');
        if ($rundesc['parameter'] && $this->FileExists($file)) {
            $writer->startElement('transmission');
            $writer->writeAttribute('run', $run->GetName());
            $writer->writeAttribute('parameter', $rundesc['parameter']);
            $writer->WriteImage($file, 'Transmission');
            $writer->endElement();
        }

        $file->SetName('events_time');
        if ($this->FileExists($file)) {
            $writer->startElement('countrate');
            $writer->WriteImage($file, "Event rate over time");
            $writer->endElement();
        }

        $writer->endElement();

        $this->WriteRunDownloads($writer, $run);

        $writer->endElement();
    }

    private function WriteRunDownloads(KDBXmlWriter $writer,
                                       KDBRunIdentifier $run)
    {
        $writer->startElement('downloads');

        $file = new KDBFileIdentifier($run, kTypeOrca);
        if ($this->FileExists($file)) {
            $writer->WriteFile($file, 'Raw Orca file', 0);
        }

        $file->SetFileType(kTypeOrcaRoot);
        if ($this->FileExists($file)) {
            $writer->WriteFile($file, 'Raw OrcaRoot trees', 0);
        }

        $file->SetFileType(kTypeCrun);
        $file->SetHardware(kHardwareIpe3);
        if ($this->FileExists($file)) {
            $writer->WriteFile($file, 'Events in crun format (IPE3 data)');
        }
        $file->SetHardware(kHardwareIpe4);
        if ($this->FileExists($file)) {
            $writer->WriteFile($file, 'Events in crun format (IPE4 data)');
        }
        $file->SetHardware(kHardwareUw);
        if ($this->FileExists($file)) {
            $writer->WriteFile($file, 'Events in crun format (UW data)');
        }

        $writer->endElement();
    }

    private function WriteSubrunInfo(KDBXmlWriter $writer,
                                     KDBRunIdentifier $run, array $subrun)
    {
        $writer->startElement('subrun');

        $identifier = clone $run;
        $identifier->SetSubrun($subrun['number']);

        $writer->writeElement('identifier', $identifier->GetName());
        $writer->writeElement('start', $subrun['start']);
        $writer->writeElement('end', $subrun['end']);
        $this->WriteParameters($writer, $subrun['parameters']);
        $writer->writeElement('comment', $subrun['comment']);

        $writer->startElement('plots');
        $writer->writeAttribute('title', 'IPE boards');

        $file = new KDBFileIdentifier($identifier, kTypePlot);
        $file->SetHardware(kHardwareIpe3);
        $file->SetName('all_rate');
        if ($this->FileExists($file)) {
            $writer->WriteImage($file, 'Rate');
        }
        $file->SetName('all_energy_log');
        if ($this->FileExists($file)) {
            $writer->WriteImage($file, 'Energy log');
        }
        $file->SetName('all_energy');
        if ($this->FileExists($file)) {
            $writer->WriteImage($file, 'Energy');
        }
        $file->SetName('all_pxdis_2d');
        if ($this->FileExists($file)) {
            $writer->WriteImage($file, 'Pixel 2D');
        }
        $file->SetName('all_pxdis_3d');
        if ($this->FileExists($file)) {
            $writer->WriteImage($file, 'Pixel 3D');
        }

        $writer->endElement();

        $writer->endElement();
    }

    private function WriteParameters(KDBXmlWriter $writer, array $parameters)
    {
        $writer->startElement('parameters');

        if (array_key_exists('magnet_west', $parameters)) {
            $value = $parameters['magnet_west'];
            $writer->WriteFloat('magnet_west', $value, 1);

            $field = $value / 157. * 4.5;
            $writer->WriteFloat('field_west', $value, 2);
        }

        if (array_key_exists('magnet_east', $parameters)) {
            $value = $parameters['magnet_east'];
            $writer->WriteFloat('magnet_east', $value, 1);

            $field = $value / 157. * 4.5;
            $writer->WriteFloat('field_east', $value, 2);
        }

        if (array_key_exists('det_ps', $parameters)) {
            $value = $parameters['det_ps'];
            $string = $value ? 'open' : 'closed';
            $writer->writeElement('det_ps', $string);
        }

        if (array_key_exists('t', $parameters)) {
            $value = $parameters['t'];
            $writer->writeElement('t', (int) $value);
        }

        if (array_key_exists('s1', $parameters)) {
            $value = $parameters['s1'];
            $writer->writeElement('s1', (int) $value);
        }

        if (array_key_exists('s2', $parameters)) {
            $value = $parameters['s2'];
            $writer->writeElement('s2', (int) $value);
        }

        if (array_key_exists('s3', $parameters)) {
            $value = $parameters['s3'];
            $writer->writeElement('s3', (int) $value);
        }

        if (array_key_exists('s4', $parameters)) {
            $value = $parameters['s4'];
            $writer->writeElement('s4', (int) $value);
        }

        if (array_key_exists('d1', $parameters)) {
            $value = $parameters['d1'];
            $writer->writeElement('d1', (int) $value);
        }

        if (array_key_exists('d2', $parameters)) {
            $value = $parameters['d2'];
            $writer->writeElement('d2', (int) $value);
        }

        if (array_key_exists('fug1', $parameters)) {
            $value = $parameters['fug1'];
            $writer->WriteFloat('fug1', $value, 3);
        }

        if (array_key_exists('fug2', $parameters)) {
            $value = $parameters['fug2'];
            $writer->WriteFloat('fug2', $value, 3);
        }

        if (array_key_exists('fug3', $parameters)) {
            $value = $parameters['fug3'];
            $writer->WriteFloat('fug3', $value, 3);
        }

        if (array_key_exists('fug4', $parameters)) {
            $value = $parameters['fug4'];
            $writer->WriteFloat('fug4', $value, 3);
        }

        if (array_key_exists('fug5', $parameters)) {
            $value = $parameters['fug5'];
            $writer->WriteFloat('fug5', $value, 3);
        }

        if (array_key_exists('pressure', $parameters)) {
            $value = $parameters['pressure'];
            $writer->WriteFloat('pressure', $value, 2, TRUE);
        }

        if (array_key_exists('ps_egun', $parameters)) {
            $value = $parameters['ps_egun'];
            $string = $value ? 'open' : 'closed';
            $writer->writeElement('ps_egun', $string);
        }

        if (array_key_exists('egun_uv', $parameters)) {
            $value = $parameters['egun_uv'];
            $string = $value ? 'on' : 'off';
            $writer->writeElement('egun_uv', $string);
        }

        if (array_key_exists('egun_shutter', $parameters)) {
            $value = $parameters['egun_shutter'];
            $string = $value ? $this->FormatFloat($value, 1) : 'closed';
            $writer->WriteElement('egun_shutter', $string);
        }

        if (array_key_exists('egun_hv', $parameters)) {
            $value = $parameters['egun_hv'];
            $writer->WriteFloat('egun_hv', $value, 2);
        }

        if (array_key_exists('egun_x', $parameters)) {
            $value = $parameters['egun_x'];
            $writer->WriteFloat('egun_x', $value, 2);
        }

        if (array_key_exists('egun_y', $parameters)) {
            $value = $parameters['egun_y'];
            $writer->WriteFloat('egun_y', $value, 2);
        }

        $writer->endElement();
    }

    private function GetDirectory(KDBFileIdentifier $file)
    {
        switch ($file->GetFileType()) {
            case kTypeOrca:
                if ($this->fPathOrca) {
                    return $this->fPathOrca;
                }
                break;
                
            case kTypeOrcaRoot:
                if ($this->fPathOrcaRoot) {
                    return $this->fPathOrcaRoot;
                }
                break;
                
            case kTypeCrun:
                if ($this->fPathCrun) {
                    return $this->fPathCrun;
                }
                break;
                
            case kTypePlot:
            case kTypePreview:
                if ($this->fPathPlots) {
                    return $this->fPathPlots;
                }
                break;
        }

        if ($file->GetRun()->IsValid()) {
            $run_name = $file->GetRun()->GetRunName();
            return $this->fFileServerRoot . '/' . $run_name;
        }
        else {
            return $this->fFileServerRoot;
        }
    }

    private function ParseSearch($string)
    {
        $pattern =
            '=^(?<system>[a-z]+)?(?<from>\d+)(-(?<to>\d+))? *(?<parameters>.*)=';

        $string = strtolower($string);

        if (!preg_match($pattern, $string, $matches)) {
            return array();
        }

        $filter = array();
        try {
            $filter['system'] =
                KDBConstants::OrcaSystemValue($matches['system']);
        }
        catch (ADEIException $e) {
        }
        $filter['from'] = $matches['from'];
        $filter['to'] = $matches['to'];
        $filter['search'] = $string;
        $filter['parameters'] = $matches['parameters'];

        return $filter;
    }

    private function DateString($date)
    {
        if (!$date or $date == 'all') {
            return '';
        }

        if ($date > 9999) {
            if ($date > 999999) {
                $day = $date % 100;
                $month = ($date % 10000) / 100;
                $year = $date / 10000;

                $tstamp = gmmktime(0, 0, 0, $month, $day, $year);
                return gmdate('Y-m-d', $tstamp);
            }
            else {
                $month = $date % 100;
                $year = $date / 100;

                $tstamp = gmmktime(0, 0, 0, $month, 1, $year);
                return gmdate('F Y', $tstamp);
            }
        }
        else {
            return "$date";
        }
    }
}

?>
