<?php
// $Id: kdbrunidentifier.php 8097 2010-07-15 09:57:24Z s_voec01 $
// Author: Sebastian Voecking <sebastian.voecking@uni-muenster.de>

ADEI::RequireClass('KDBConstants', TRUE);

class KDBRunIdentifier
{
    private $fSystem = kSystemInvalid;
    private $fRun = 0;
    private $fSubrun = 0;
    
    function __construct()
    {
        if (func_num_args() == 1 && is_string(func_get_arg(0))) {
            $this->ParseString(func_get_arg(0));
        }
        else {
            if (func_num_args() >= 1) {
                $this->fSystem = func_get_arg(0);
            }
            if (func_num_args() >= 2) {
                $this->fRun = func_get_arg(1);
            }
            if (func_num_args() >= 3) {
                $this->fSubrun = func_get_arg(2);
            }
        }
    }

    function GetRun()
    {
        return $this->fRun;
    }

    function GetSubRun()
    {
        return $this->fSubrun;
    }

    function SetSubRun($subrun)
    {
        $this->fSubrun = $subrun;
    }

    function GetSystem()
    {
        return $this->fSystem;
    }

    function GetRunName()
    {
        return sprintf("%s%08d", $this->GetSystemName(), $this->fRun);
    }

    function GetName()
    {
        $name = $this->GetRunName();
        if ($this->fSubrun != 0) {
            $name .= sprintf(".%05d", $this->fSubrun);
        }

        return $name;
    }

    function IsValid()
    {
        return $this->fSystem != kSystemInvalid && $this->fRun != 0;
    }

    private function ParseString($text)
    {
        $pattern = '=^(?<system>[a-z]+)(?<run>\d+)(\.(?<subrun>\d+))?=';
        
        if (preg_match($pattern, $text, $matches)) {
            try {
                $this->fSystem =
                    KDBConstants::OrcaSystemValue($matches['system']);
            }
            catch (ADEIException $e) {
                $this->fSystem = kSystemInvalid;
            }
            
            $this->fRun = (int) $matches['run'];
            if (array_key_exists('subrun', $matches)) {
                $this->fSubrun = (int) $matches['subrun'];
            }
        }
    }

    private function GetSystemName()
    {
        switch ($this->fSystem) {
            case kSystemMainSpec:
                return 'kms';
            case kSystemPreSpec:
                return 'kps';
            default:
                return 'xxx';
        }
    }
}

?>
