<?php
// $Id$
// Author: Sebastian Voecking <sebastian.voecking@uni-muenster.de>

class KDBSql
{
    private $fMysql = NULL;
    private $fHostname;
    private $fDatabase;
    private $fUser;
    private $fPassword;
    
    function __construct($hostname, $database, $user, $password)
    {
        $this->fHostname = $hostname;
        $this->fDatabase = $database;
        $this->fUser = $user;
        $this->fPassword = $password;
    }

    function ListRuns(array $filter, &$total)
    {
        $this->Connect();

        $where = $this->BuildFilter($filter);

        if (array_key_exists('limit', $filter)) {
            if (array_key_exists('offset', $filter)) {
                $limit = "{$filter['offset']}, {$filter['limit']}";
            }
            else {
                $limit = $filter['limit'];
            }
        }

        $query = "SELECT SQL_CALC_FOUND_ROWS run.system AS system, ";
        $query .= "run.number AS number, run.start AS start, ";
        $query .= "run.end AS end, ";
        $query .= "TIMEDIFF(run.end, run.start) AS duration, ";
        $query .= "run.comment AS comment, run.data_types AS data, ";
        $query .= "COUNT(*) AS subruns FROM run, subrun";

        $query .= ' WHERE subrun.run = run.id';
        if ($where) {
            $query .= ' AND ' . implode(' AND ', $where);
        }
        $query .= ' GROUP BY subrun.run ORDER BY run.start';
        if ($limit) {
            $query .= " LIMIT $limit";
        }

        $result = mysql_query($query, $this->fMysql);

        $runs = array();
        while ($row = mysql_fetch_assoc($result)) {
            $runs[] = $row;
        }

        $query = "SELECT FOUND_ROWS()";
        $result = mysql_query($query, $this->fMysql);
        $row = mysql_fetch_array($result);
        $total = $row[0];

        return $runs;
    }

    function GetRunDescription(KDBRunIdentifier $run, $subruns)
    {
        $this->Connect();

        $system = $run->GetSystem();
        $run_number = $run->GetRun();

        $query = 'SELECT run.id AS id, CAST(run.start AS DATE) AS date, ';
        $query .= 'CAST(run.start AS TIME) as start, ';
        $query .= 'CAST(run.end AS TIME) as end, ';
        $query .= 'run.orca_configuration AS configuration, ';
        $query .= 'run.orca_version AS orca_version, ';
        $query .= 'run.comment AS comment, run.data_types AS streams, ';
        $query .= 'parameter.name AS parameter ';
        $query .= 'FROM run LEFT JOIN parameter ON ';
        $query .= 'run.control_parameter = parameter.id WHERE ';
        $query .= "system = $system AND number = $run_number";

        $result = mysql_query($query, $this->fMysql);
        $rundesc = mysql_fetch_assoc($result);

        if ($rundesc) {
            $rundesc['parameters'] = $this->GetParameters($run);
            if ($subruns) {
                $rundesc['subruns'] =
                    $this->GetSubrunDescriptions($run, $rundesc['id']);
            }
        }

        return $rundesc;
    }

    function GetSubrunDescriptions(KDBRunIdentifier $run, $id)
    {
        $this->Connect();

        $query = 'SELECT subrun.number AS number, ';
        $query .= 'TIMEDIFF(subrun.start, run.start) AS start, ';
        $query .= 'TIMEDIFF(subrun.end, run.start) AS end, ';
        $query .= 'subrun.comment AS comment FROM run, subrun WHERE ';
        $query .= "subrun.run = $id AND run.id = $id ORDER BY subrun.number";

        $result = mysql_query($query, $this->fMysql);
        $subruns = array();

        while ($subrun = mysql_fetch_assoc($result)) {
            $subrun_identifier = clone $run;
            $subrun_identifier->SetSubRun($subrun['number']);
            $subrun['parameters'] = $this->GetParameters($subrun_identifier);
            $subruns[] = $subrun;
        }

        return $subruns;
    }

    function GetParameters(KDBRunIdentifier $run)
    {
        $this->Connect();

        $system = $run->GetSystem();
        $run_number = $run->GetRun();
        $subrun_number = $run->GetSubrun();

        if ($run->GetSubRun() == 0) {
            $query = "SELECT parameter.name, run_parameter.value FROM ";
            $query .= "parameter, run_parameter WHERE ";
            $query .= "run_parameter.run = (SELECT id FROM run WHERE ";
            $query .= "system = $system AND number = $run_number) AND ";
            $query .= "parameter.id = run_parameter.parameter";
        }
        else {
            $query = "SELECT parameter.name, subrun_parameter.value FROM ";
            $query .= "parameter, subrun_parameter WHERE ";
            $query .= "subrun_parameter.subrun = (SELECT id FROM subrun WHERE ";
            $query .= "run = (SELECT id FROM run WHERE ";
            $query .= "system = $system AND number = $run_number) AND ";
            $query .= "number = $subrun_number) AND ";
            $query .= "parameter.id = subrun_parameter.parameter";
        }

        if(!($result = mysql_query($query, $this->fMysql))) {
            $f = fopen("php://stderr", "w");
            fwrite($f, "$query\n");
            fclose($f);
        }

        $parameters = array();
        while ($row = mysql_fetch_array($result)) {
            $parameters[$row[0]] = (float) $row[1];
        }

        return $parameters;
    }
    
    function GetDates(array $filter)
    {
        $this->Connect();

        $where = $this->BuildFilter($filter);

        $query = 'SELECT UNIX_TIMESTAMP(CAST(start AS DATE)) AS d FROM run';
        if ($where) {
            $query .= ' WHERE ' . implode(' AND ', $where);
        }
        $query .= ' GROUP BY d ORDER BY d';
        $result = mysql_query($query, $this->fMysql);

        $dates = array();
        while ($row = mysql_fetch_row($result)) {
            $dates[] = $row[0];
        }

        return $dates;
    }

    function HaveRuns($from, $to)
    {
        $this->Connect();

        $interval = "BETWEEN FROM_UNIXTIME($from) AND FROM_UNIXTIME($to)";

        $query = "SELECT id FROM run WHERE ";
        $query .= "start $interval OR end $interval LIMIT 1";

        $result = mysql_query($query, $this->fMysql);
        return (bool) mysql_fetch_array($result);
    }

    function HaveCountRates($from, $to)
    {
        $this->Connect();

        $interval = "BETWEEN FROM_UNIXTIME($from) AND FROM_UNIXTIME($to)";

        $query = "SELECT time FROM countrates_prespec WHERE ";
        $query .= "time $interval LIMIT 1";

        $result = mysql_query($query, $this->fMysql);
        return (bool) mysql_fetch_array($result);
        
    }

    function GetRunData($from, $to)
    {
        $this->Connect();

        $interval = "BETWEEN FROM_UNIXTIME($from) AND FROM_UNIXTIME($to)";

        $query = 'SELECT run.number AS run, subrun.number AS subrun, ';
        $query .= 'UNIX_TIMESTAMP(subrun.start) AS start, ';
        $query .= 'UNIX_TIMESTAMP(subrun.end) AS end ';
        $query .= 'total.subruns AS subruns ';
        $query .= 'FROM run, subrun, ';
        $query .= '(SELECT run, count(*) AS subruns FROM subrun GROUP BY run) ';
        $query .= 'AS total ';
        $query .= 'WHERE run.id = subrun.run AND total.run = subrun.run ';
        $query .= "(subrun.start $interval OR end $interval) ";
        $query .= 'ORDER BY subrun.start';

        return mysql_query($query, $this->fMysql);
    }

    function GetCountRates($from, $to, $hardware, array $items)
    {
        $this->Connect();

        switch ($hardware) {
            case kHardwareUw:
                $hardwarestr = 'uw';

            case kHardwareIpe3:
                $hardwarestr = 'ipe3';

            case kHardwareIpe4:
                $hardwarestr = 'ipe4';
        }
        
        $interval = "BETWEEN FROM_UNIXTIME($from) AND FROM_UNIXTIME($to)";
        
        $query = 'SELECT UNIX_TIMESTAMP(time)';
        
        foreach ($items as $item) {
            if ($item == 0) {
                $query .= ', total';
            }
            else {
                $query .= ", pixel_$item";
            }
        }

        $query .= " FROM countrates_prespec WHERE board = '$hardware' ";
        $query .= "AND time $interval ORDER BY time";

        return mysql_query($query, $this->fMysql);
    }

    private function Connect()
    {
        if ($this->fMysql) return;

        $this->fMysql = mysql_connect($this->fHostname, $this->fUser,
                                      $this->fPassword);
        if ($this->fMysql) {
            mysql_select_db($this->fDatabase, $this->fMysql);
        }
        mysql_query("SET time_zone = '+00:00'", $this->fMysql);
    }

    private function ConvertTimeStamp($tstamp)
    {
        return "FROM_UNIXTIME(" . $tstamp . ")";
    }

    private function BuildFilter($filter)
    {
        $where = array();
        if (array_key_exists('date', $filter)) {
            $date = $filter['date'];
            if ($date != 0 && $date != "all") {
                if ($date > 999999) {
                    $start = $date;
                    $end = "$start + INTERVAL 1 DAY";
                }
                elseif ($date > 9999) {
                    $start = "${date}01";
                    $end = "$start + INTERVAL 1 MONTH";
                }
                else {
                    $start = "${date}0101";
                    $end_date = $date + 1;
                    $end = "${end_date}0101";
                }
                $where[] =
                    "run.start BETWEEN $start AND $end - INTERVAL 1 SECOND";
            }
        }

        if (array_key_exists('system', $filter)) {
            $where[] = "run.system = {$filter['system']}";
        }

        $from = $filter['from'];
        $to = $filter['to'];

        if ($from) {
            if ($to) {
                $where[] = "run.number BETWEEN $from AND $to";
            }
            else {
                $where[] = "run.number = $from";
            }
        }
        return $where;
    }
}

?>
