<?php
// $Id$
// Author: Sebastian Voecking <sebastian.voecking@uni-muenster.de>

class KDBXmlWriter extends XMLWriter
{
    function __construct()
    {
        $this->openMemory();
        $this->setIndent(true);
        $this->setIndentString(' ');
        $this->startDocument();
    }

    function WriteFloat($name, $value, $digits, $scientific = FALSE)
    {
        $format = $scientific ? 'e' : 'f';
        $string = sprintf("%.$digits$format", $value);
        $this->writeElement($name, $string);
    }

    function WriteStreams($streams)
    {
        $names = array("uw", "ipe3_energy", "ipe3_waveform", "ipe3_histogram",
                       "ipe4_energy", "ipe4_waveform", "ipe4_histogram");

        $this->startElement("streams");
        for ($i = 0; $i < 7; $i++) {
            if ($streams & (1 << $i)) {
                $this->startElement("type");
                $this->writeAttribute("name", $names[$i]);
                $this->endElement();
            }
        }
        $this->endElement();
    }

    function WriteImage(KDBFileIdentifier $file, $description)
    {
        $this->startElement('image');

        if ($file->GetRun()->IsValid()) {
            $this->writeAttribute('run', $file->GetRun()->GetName());
        }

        $this->writeAttribute('name', $file->GetName());
        if ($file->GetHardware() != kHardwareInvalid) {
            $this->writeAttribute('hardware', $file->GetHardwareName());
        }

        $this->text($description);

        $this->endElement();
    }

    function WriteFile(KDBFileIdentifier $file, $description, $filesize)
    {
        $type_names = array('orca', 'orcaroot', 'crun', 'header', 'plot',
                            'preview', 'custom');

        $this->startElement('file');

        if ($file->GetRun()->IsValid()) {
            $this->writeAttribute('run', $file->GetRun()->GetName());
        }

        if ($file->GetFileType() != kTypeCustom) {
            $this->writeAttribute('type', $type_names[$file->GetFileType()]);
        }

        if ($file->GetHardware() != kHardwareInvalid) {
            $this->writeAttribute('hardware', $file->GetHardwareName());
        }

        if ($file->GetName()) {
            $this->writeAttribute('name', $file->GetName());
        }

        if ($filesize > 0) {
            $quantity = new KDBQuantity($filesize, 'B');
            $quantify->SetPrecision(-3);
            $this->writeAttribute('filesize', $quantity);
        }

        $this->text($description);

        $this->endElement();
    }
}
?>
