<?php
// $Id: katrinreader.php 8051 2010-07-08 12:37:47Z s_voec01 $
// Author: Sebastian Voecking <sebastian.voecking@uni-muenster.de>

ADEI::RequireClass("KDBPhp", true);

class KATRINCountrates implements Iterator
{
    private $fData;
    private $fRow;
    
    function __construct(KDBPhp $kdb, OPTIONS &$opts, array &$items,
        INTERVAL &$ivl, $hardware, $resample)
    {
        $from = $ivl->GetWindowStart();
        $to = $ivl->GetWindowEnd();
        $this->fData =
            $kdb->GetSql()->GetCountRates($from, $to, $hardware, $items);
        $this->next();
    }
    
    function rewind()
    {
        mysql_data_seek($this->fData, 0);

        $this->next();
    }

    function current()
    {
        if ($this->valid()) {
            return array_slice($this->fRow, 1, count($this->fRow) - 1);
        }
        else {
            return FALSE;
        }
    }

    function next()
    {
        $this->fRow = mysql_fetch_array($this->fData);

        return $this->valid();
    }

    function valid()
    {
        return $this->fRow;
    }

    function key()
    {
        if ($this->valid()) {
            return $this->fRow[0];
        }
        else {
            return FALSE;
        }
    }
}

class KATRINRuns implements Iterator
{
    private $fData;
    private $fSubrunEnd = TRUE;
    private $fRow;
    private $fItems;
    
    function __construct(KDBPhp &$kdb, OPTIONS &$opts, array &$items,
        INTERVAL &$ivl, $resample)
    {
        $from = $ivl->GetWindowStart();
        $to = $ivl->GetWindowEnd();
        $this->fItems = $items;
        $this->fData = $kdb->GetSql()->GetRunData($from, $to);
        $this->next();
    }
    
    function rewind()
    {
        mysql_data_seek($this->fData, 0);
        $this->fSubrunEnd = TRUE;
        $this->next();
    }

    function current()
    {
        $values = array();
        foreach ($this->fItems as $item) {
            switch ($item) {
                case 0:
                    $value = $this->fRow['run'];
                    $value +=
                        $this->fRow['subrun'] / (float) $this->fRow['subruns'];
                    $values[] = $value;
                    break;
                    
                case 1:
                    $values[] = $this->fRow['run'];
                    break;
                    
                case 2:
                    $values[] = 1;
                    break;
                    
                default:
                    $values[] = 0;
            }
        }
    }

    function next()
    {
        if ($this->fSubrunEnd) {
            $this->fRow = mysql_fetch_assoc($this->fData);
        }
        $this->fSubrunEnd = !$this->fSubrunEnd;

        return $this->valid();
    }

    function valid()
    {
        return $this->fRow;
    }

    function key()
    {
        if ($this->valid()) {
            if ($this->fSubrunEnd) {
                return $this->fRow['end'];
            }
            else {
                return $this->fRow['start'];
            }
        }
    }
}

abstract class KATRINData implements Iterator
{
    public $_cPtr = NULL;
    private $iterator = NULL;

    function rewind()
    {
        try {
            KDBDataIter_Rewind($this->_cPtr);
        }
        catch (Exception $e) {
            throw new ADEIException($e->getMessage());
        }
    }

    function current()
    {
        try {
            $r = KDBDataIter_Current($this->_cPtr);
            $a = vectord_to_array($r);
            return $a;
        }
        catch (Exception $e) {
            throw new ADEIException($e->getMessage());
        }
    }

    function next()
    {
        try {
            return KDBDataIter_Next($this->_cPtr);
        }
        catch (Exception $e) {
            throw new ADEIException($e->getMessage());
        }
    }

    function valid()
    {
        try {
            return KDBDataIter_Valid($this->_cPtr);
        }
        catch (Exception $e) {
            throw new ADEIException($e->getMessage());
        }
    }

    function key()
    {
        try {
            return KDBDataIter_Key($this->_cPtr);
        }
        catch (Exception $e) {
            throw new ADEIException($e->getMessage());
        }
    }
}

class KATRINReader extends READER
{
    public $kdb = NULL;

    function __construct(&$props)
    {
        parent::__construct($props);

        $this->kdb = new KDBPhp();

        $items = array();
        $items[] = array('name' => "Total");
        for ($i = 1; $i < 65; $i++) {
            $name = "Pixel " . $i;
            $items[] = array('name' => $name);
        }

        $this->items = array();
        $this->items['countrates_uw'] = $items;
        $this->items['countrates_ipe3'] = $items;
        $this->items['countrates_ipe4'] = $items;

        $this->items['runs'] = array();
        $this->items['runs'][] = array('name' => "Run numbers");
        $this->items['runs'][] = array('name' => "Run numbers (no subruns)");
        $this->items['runs'][] = array('name' => "Activity");

        $this->groups = array(
            'countrates_uw' => array('name' => "Pixel Countrates (UW)"),
            'countrates_ipe3' => array('name' => "Pixel Countrates (IPE3)"),
            'countrates_ipe4' => array('name' => "Pixel Countrates (IPE4)"),
            'runs' => array('name' => "Runs")
        );
    }

    function GetGroupInfo(LOGGROUP $grp = NULL, $flags = 0)
    {
        $groups = array();

        foreach ($this->groups as $gid => &$group) {
            if ($grp && strcmp($grp->gid, $gid)) continue;

            if ($group['name']) $name = $group['name'];

            if (!$name || ($flags & REQUEST::NEED_INFO)) {
                if ($grp) {
                    $grtest = $grp;
                    $opts = $this->opts;
                }
                else {
                    $ginfo = array("db_group" => $gid);
                    $grtest = $this->CreateGroup($ginfo);
                    $opts = $this->req->GetGroupOptions($grtest);
                }

                if (!$name) {
                    $name = $opts->Get('name', $gid);
                }
            }

            $groups[$gid] = array('gid' => $gid, 'name' => $name);

            if ($flags & REQUEST::NEED_INFO) {
                $limit = $opts->GetDateLimit(0, time());
                $groups[$gid]['first'] = $limit[0];
                $groups[$gid]['last'] = $limit[1];

                if ($flags & REQUEST::NEED_COUNT) {
                    $period = $opts->Get('period');
                    $groups[$gid]['records'] =
                        ($groups[$gid]['last'] - $groups[$gid]['first']) /
                        $period;
                }

                if ($flags & REQUEST::NEED_ITEMINFO) {
                    $groups[$gid]['items'] = $this->GetItemList($grtest);
                }
            }
        }

        if ($grp && !$groups) {
            $error = translate("Invalid group (%s) is specified", $grp->gid);
            throw new ADEIException($error);
        }

        return $grp ? $groups[$grp->gid] : $groups;
    }

    function GetItemList(LOGGROUP $grp = NULL, MASK $mask = NULL, $flags = 0)
    {
        if ($flags & REQUEST::ONLY_AXISINFO) {
            if (!$this->req->GetGroupOptions($grp, "axis")) return array();
        }

        $grp = $this->CheckGroup($grp);
        if (!$mask) $mask = $this->CreateMask($grp);

        $items = array();

        $opts = $this->req->GetGroupOptions($grp);
        $items = $opts->Get('items', $this->items[$grp->gid]);

        $res = array();

        for ($i = 0; isset($items[$i]); $i++) {
            if (!$mask->Check($i)) continue;

            $item = array("id" => $i, "name" => $items[$i]['name']);

            if ($items[$i]['uid']) {
                $item['uid'] = $items[$i]['uid'];
            }

            if ($flags & REQUEST::NEED_AXISINFO) {
                if ($grp->GetGroupID() == "countrates_uw" ||
                    $grp->GetGroupID() == "countrates_ipe3" ||
                    $grp->GetGroupID() == "countrates_ipe4") {
                    $item["axis"] = "frequency";
                }
                elseif ($grp->GetGroupID() == "runs") {
                    $item["axis"] = 0;
                }
            }

            $res[] = $item;
        }

        return $res;
    }

    function GetMaskList(LOGGROUP $grp = NULL, $flags = 0)
    {
        //print "KATRINReader::GetMaskList";

        $grp = $this->CheckGroup($grp);

        $masks = array();

        if ($grp->GetGroupID() == "runs") {
        }
        else if ($grp->GetGroupID() == "countrates_uw" ||
                 $grp->GetGroupID() == "countrates_ipe3" ||
                 $grp->GetGroupID() == "countrates_ipe4") {
            $mask = array();
            $mask["id"] = "total";
            $mask["name"] = "Total";
            if ($flags & REQUEST::NEED_INFO) {
                $mask["mask"] = "0";
            }
            $masks[] = $mask;

            $mask = array();
            $mask["id"] = "ipe";
            $mask["name"] = "Pixels";
            if ($flags & REQUEST::NEED_INFO) {
                $items = array();
                for ($i = 1; $i < 65; $i++) {
                    $items[] = $i;
                }
                $mask["mask"] = implode(",", $items);
            }
            $masks[] = $mask;
        }

        return array_merge(parent::GetMaskList($grp, $flags), $masks);
    }

    function GetRawData(LOGGROUP $grp = NULL, $from = 0, $to = 0,
        DATAFilter $filter = NULL, &$filter_data = NULL)
    {
        $grp = $this->CheckGroup($grp);

        $ivl = $this->CreateInterval($grp);
        $ivl->Limit($from, $to);

        if ($filter) {
            $mask = $filter->GetItemMask();
            $resample = $filter->GetSamplingRate();
            $limit = $filter->GetVectorsLimit();
            if ($limit) $ivl->SetItemLimit($limit);

            if (isset($filter_data)) {
                if ($mask) $filter_data['masked'] = true;
                if ($resample) $filter_data['resampled'] = true;
                if ($limit) $filter_data['limited'] = true;
            }
        }
        else {
            $mask = NULL;
            $resample = 0;
            $limit = 0;
        }

        $opts = $this->req->GetGroupOptions($grp);

        $items = $opts->Get('items', $this->items[$grp->gid]);
        if ($mask && is_array($ids = $mask->GetIDs())) {
            $tmp = array();
            foreach ($ids as $id) {
                if ($id >= sizeof($items)) {
                    $error =
                        translate("Invalid item mask is supplied. " .
                        "The ID:%d refers non-existing item.", $id);
                    throw new ADEIException($error);
                }
                array_push($tmp, $items[$id]);
            }
            $items = $tmp;
        }

        if ($grp->GetGroupID() == "countrates_uw") {
            return new KATRINCountrates($this->kdb,
                $this->req->GetGroupOptions($grp), $items, $ivl, kHardwareUw,
                $resample);
        }
        else if ($grp->GetGroupID() == "countrates_ipe3") {
            return new KATRINCountrates($this->kdb,
                $this->req->GetGroupOptions($grp), $items, $ivl, kHardwareIpe3,
                $resample);
        }
        else if ($grp->GetGroupID() == "countrates_ipe4") {
            return new KATRINCountrates($this->kdb,
                $this->req->GetGroupOptions($grp), $items, $ivl, kHardwareIpe4,
                $resample);
        }
        else if ($grp->GetGroupID() == "runs") {
            return new KATRINRuns($this->kdb, $this->req->GetGroupOptions($grp),
                $items, $ivl, $resample);
        }
        else {
            return NULL;
        }
    }

    function HaveData(LOGGROUP $grp = NULL, $from = 0, $to = 0)
    {
        if (is_null($grp) || $grp->GetGroupID() == "runs") {
            return $this->kdb->HaveRuns($from, $to);
        }
        else if ($grp->GetGroupID() == "countrates_uw" ||
                 $grp->GetGroupID() == "countrates_ipe3" ||
                 $grp->GetGroupID() == "countrates_ipe4") {
            return $this->kdb->HaveCountrates($from, $to);
        }
        else {
            return FALSE;
        }
    }
}

?>
