<?php
// Author: Sebastian Voecking <sebastian.voecking@uni-muenster.de>

ADEI::RequireClass('KDBConstants', TRUE);
ADEI::RequireClass('KDBPhp', TRUE);
ADEI::RequireClass('KDBRunIdentifier', TRUE);

class KDBSearch extends SEARCHEngine
{
    function __construct(REQUEST $req = NULL, $opts = FALSE)
    {
        parent::__construct($req, $opts);
        $this->modules = array('runs' => _('Runs'));
        $this->engine_search = array('runs');
    }

    function DetectModules($string)
    {
        $pos = strpos($string, ' ');
        if ($pos !== FALSE) {
            $word = substr($string, $pos);
        }
        else {
            $word = $string;
        }

        $word = strtolower($word);

        if ($word == 'run' or $word == 'runs') {
            return array('runs' => TRUE);
        }
        else {
            return FALSE;
        }
    }

    function Search($search_string, $module, SEARCHFilter $filter = NULL,
                    $opts = FALSE)
    {
        $pattern = '=^(runs|run) ?(?<search>.*)=';

        $kdb = new KDBPhp();

        if (!preg_match($pattern, strtolower($search_string), $matches)) {
            return FALSE;
        }

        $runs = $kdb->SearchRuns($matches['search']);
        $number = count($runs);
        
        $results = new SEARCHResults($filter, $this, $module);
        $item = array(
            'title' => 'Result overview',
            'props' => array(
                'kdb_run' => '',
                'kdb_search' => $matches['search'],
                'kdb_page' => 1,
                'module' => 'katrin',
                'kdb_date' => ''),
            'certain' => ($number != 1));
        if ($number == 1) {
            $description = "1 run found";
        }
        else {
            $description = "$number runs found";
        }
        $item['description'] =
            "<div class=\"description\">$description</div>";
        $results->Append($item);
        
        foreach ($runs as $run) {
            $item = array();
            $identifier =
                new KDBRunIdentifier($run['system'], $run['number']);
            $item['title'] = 'Run ' . $identifier->GetName();
            $item['props'] = array(
                'kdb_run' => $identifier->GetName(),
                'kdb_subruns' => 0,
                'kdb_search' => $matches['search'],
                'kdb_page' => 1,
                'module' => 'katrin',
                'kdb_date' => '');
            $description = "{$run['start']}, Duration: {$run['duration']}, ";
            if ($run['subruns'] == 1) {
                $description .= "1 subrun";
            }
            else {
                $description .= "{$run['subruns']} subruns";
            }
            $item['description'] =
                "<div class=\"description\">$description</div>";
            $item['certain'] = ($number == 1);
            $results->Append($item);
        }

        return $results;
    }
}

?>