<?php
// $Id: katrin.php 8100 2010-07-15 12:39:05Z s_voec01 $
// Author: Sebastian Voecking <sebastian.voecking@uni-muenster.de>

global $ADEI;

ADEI::RequireClass('KDBConstants', TRUE);
ADEI::RequireClass('KDBFileIdentifier', TRUE);
ADEI::RequireClass('KDBPhp', TRUE);
ADEI::RequireClass('KDBRunIdentifier', TRUE);

function PrintXml($out, $xml)
{
    header("Content-type: text/xml");
    header("Cache-Control: no-cache, must-revalidate");
    header("Expires: Mon, 26 Jul 1997 05:00:00 GMT");

    if ($out) {
        fwrite($out, $xml);
        fclose($out);
    }
    else {
        print $xml;
    }
}

function GetContentType($filename)
{
    if (substr($filename, -4) == ".png") {
        return "image/png";
    }
    else if (substr($filename, -4) == ".txt") {
        return "text/plain";
    }
    else if (substr($filename, -4) == ".xml") {
        return "application/xml";
    }
    else if (substr($filename, -4) == ".svg") {
        return "image/svg+xml";
    }
    else {
        return "application/octet-stream";
    }
}

function PrintFile($path, $download)
{
    if (!file_exists($path))
        throw new ADEIException("File $path does not exist");

    $mimetype = GetContentType($path);
    $basename = basename($path);

    header("Content-type: $mimetype");
    if ($download)
        header("Content-disposition: attachment; filename=\"$basename\"");
    readfile($path);
}

function PrintError($error)
{
    $xml = "<?xml version=\"1.0\"?>\n<result>\n";
    $xml .= " <Error>$error</Error>\n</result>";

    PrintXml(NULL, $xml);
}

$out = NULL;

try {
    $kdb = new KDBPhp();
    $req = new REQUEST();

    $target = $req->GetProp('target', NULL);
    $run = new KDBRunIdentifier($req->GetProp('kdb_run', ''));
    $xslt = $req->GetProp('xslt');
    $page = $req->GetProp('kdb_page', 1);
    $date = $req->GetProp('kdb_date', '');
    $search = urldecode($req->GetProp('kdb_search', ''));
    $parameters = urldecode($req->GetProp('kdb_parameters', ''));
    
    if ($xslt) {
        $temp_file = tempnam(sys_get_temp_dir(), 'adei_katrin.');
        $out = @fopen($temp_file, 'w');
        if (!$out) {
            throw new ADEIException("I'm not able to create temporary file \"%s\"",
                                    $temp_file);
        }
    }
    
    switch ($target) {
        case 'runs':
            $interval = $req->CreateInterval();
            $start = $interval->GetWindowStart();
            $end = $interval->GetWindowEnd();

            PrintXml($out, $kdb->ListRuns($start, $end));
            break;

        case 'info':
            if ($run->IsValid()) {
                PrintXml($out, $kdb->GetRun($_GET['run']));
            }
            else {
                throw new ADEIException("No run specified");
            }
            break;

        case 'rundesc':
            if (!$run->IsValid()) {
                PrintXml($out, $kdb->ListLastRuns($date, $search, $parameters,
                         $page, 30));
                break;
            }

            $subruns = $req->GetProp("kdb_subruns", false);
            PrintXml($out, $kdb->GetRunDescription($run, $subruns));
            break;

        case 'file':
            $type = KDBConstants::FileTypeValue($req->GetProp('kdb_type'));
            $file = new KDBFileIdentifier($run, $type);
            $file->SetName($req->GetProp('kdb_name', ''));

            $hardware = $req->GetProp('kdb_hardware', NULL);
            if (isset($hardware)) {
                $file->SetHardware(KDBConstants::HardwareValue($hardware));
            }

            PrintFile($kdb->GetAbsolutePath($file), $download);
            break;

        case 'dates':
            PrintXml($out, $kdb->GetDates($search));
            break;

        default:
            if (isset($target)) {
                $message = "Unknown target ($target) is specified";
                throw new ADEIException($message);
            }
            else {
                throw new ADEIException('The target is not specified');
            }
    }
}
catch (ADEIException $e) {
    PrintError($e->getMessage());
}

if ($temp_file) {
    echo $ADEI->TransformXML($xslt, $temp_file);
    @unlink($temp_file);
}

?>
